from __future__ import print_function, unicode_literals
import inspect

import six
from django import forms
from django.forms.forms import DeclarativeFieldsMetaclass
from rest_framework import serializers

from .. import fields
from ..utils import (
    initialize_class_using_reference_object,
    reduce_attr_dict_from_base_classes,
)
from .fields import ISO8601DateTimeField


SERIALIZER_FORM_FIELD_MAPPING = {
    fields.BooleanField: forms.BooleanField,
    fields.CharField: forms.CharField,
    fields.ChoiceField: forms.ChoiceField,
    fields.DateTimeField: ISO8601DateTimeField,
    fields.EmailField: forms.EmailField,
    fields.IntegerField: forms.IntegerField,
    serializers.BooleanField: forms.BooleanField,
    serializers.CharField: forms.CharField,
    serializers.ChoiceField: forms.ChoiceField,
    serializers.DateTimeField: ISO8601DateTimeField,
    serializers.EmailField: forms.EmailField,
    serializers.IntegerField: forms.IntegerField,
}


class SerializerFormOptions(object):
    def __init__(self, options=None, name=None):
        self.serializer = getattr(options, 'serializer', None)
        self.fields = getattr(options, 'fields', [])
        self.exclude = getattr(options, 'exclude', [])
        self.field_mapping = getattr(options, 'field_mapping', {})

        assert self.serializer is not None, (
            '{}.Meta.serializer must be provided'
            ''.format(name)
        )
        assert issubclass(self.serializer, serializers.BaseSerializer), (
            '{}.Meta.serializer must be a subclass of DRF serializer'
            ''.format(name)
        )


class SerializerFormMeta(DeclarativeFieldsMetaclass):
    def __new__(cls, name, bases, attrs):
        try:
            parents = [b for b in bases if issubclass(b, SerializerForm)]
        except NameError:
            # We are defining SerializerForm itself
            parents = None

        meta = attrs.pop('Meta', None)

        if not parents or attrs.pop('_is_base', False):
            return super(SerializerFormMeta, cls).__new__(cls, name, bases, attrs)

        attrs['_meta'] = options = SerializerFormOptions(meta, name=name)

        new_attrs = cls.get_form_fields_from_serializer(bases, options)
        # attrs should take priority in case a specific field is overwritten
        new_attrs.update(attrs)

        return super(SerializerFormMeta, cls).__new__(cls, name, bases, new_attrs)

    @classmethod
    def get_field_mapping(cls, bases, options):
        mapping = reduce_attr_dict_from_base_classes(
            bases,
            lambda i: getattr(getattr(i, '_meta', None), 'field_mapping', {}),
            SERIALIZER_FORM_FIELD_MAPPING
        )
        mapping.update(options.field_mapping)
        return mapping

    @classmethod
    def get_form_fields_from_serializer(cls, bases, options):
        fields = {}

        mapping = cls.get_field_mapping(bases, options)

        for name, field in options.serializer._declared_fields.items():
            if field.read_only:
                continue

            if name not in options.fields or name in options.exclude:
                continue

            form_field_class = mapping.get(type(field))

            if not form_field_class:
                raise TypeError(
                    '{} is not mapped to appropriate form field class. '
                    'Please add it to the mapping via `field_mapping` '
                    'Meta attribute.'
                    ''.format(type(field))
                )

            fields[name] = initialize_class_using_reference_object(field, form_field_class)

        return fields


class SerializerFormBase(forms.Form):
    def __init__(self, *args, **kwargs):
        super(SerializerFormBase, self).__init__(*args, **kwargs)
        # instantiated during validation
        self.serializer = None

    def get_serializer_context(self):
        return {}

    def get_serializer_data(self):
        data = self.initial.copy()
        data.update(self.cleaned_data or {})
        return data

    def get_serializer(self):
        return self._meta.serializer(
            data=self.get_serializer_data(),
            context=self.get_serializer_context()
        )

    def _clean_form(self):
        super(SerializerFormBase, self)._clean_form()

        self.serializer = self.get_serializer()

        if not self.serializer.is_valid():
            self._errors.update(self.serializer.errors)
        else:
            self.cleaned_data = self.serializer.validated_data


class SerializerForm(six.with_metaclass(SerializerFormMeta, SerializerFormBase)):
    _is_base = True


def form_from_serializer(serializer, **kwargs):
    assert inspect.isclass(serializer) and issubclass(serializer, serializers.BaseSerializer), (
        'Can only create forms from DRF Serializers'
    )
    kwargs.update({'serializer': serializer})
    meta = type(str('Meta'), (object,), kwargs)
    return type(str('{}Form'.format(serializer.__name__)), (SerializerForm,), {'Meta': meta})



from __future__ import print_function, unicode_literals
import unittest

import mock
from rest_framework import fields, serializers

from drf_braces.serializers.enforce_validation_serializer import (
    EnforceValidationFieldMixin,
    _create_enforce_validation_serializer,
    add_base_class_to_instance,
    create_enforce_validation_serializer,
)


TESTING_MODULE = 'drf_braces.serializers.enforce_validation_serializer'


class InnerSerializer(serializers.Serializer):
    field = fields.IntegerField()
    field2 = fields.IntegerField()
    must_validate_fields = ['field']


class TestSerializer(serializers.Serializer):
    field = fields.IntegerField()
    field2 = fields.IntegerField()
    inner = InnerSerializer()


class TestManySerializer(serializers.Serializer):
    many = TestSerializer(many=True)


class TestEnforceValidationFieldMixin(unittest.TestCase):
    class Field(EnforceValidationFieldMixin, fields.IntegerField):
        pass

    def test_field_from_native_must_validate(self):
        field = self.Field()
        field.field_name = 'field'
        field.parent = mock.MagicMock(must_validate_fields=None)

        self.assertEqual(field.to_internal_value('5'), 5)

    def test_field_from_native_must_validate_all(self):
        field = self.Field()
        field.field_name = 'field'
        field.parent = mock.MagicMock(must_validate_fields=None)

        with self.assertRaises(serializers.ValidationError):
            field.to_internal_value('hello')

    def test_field_from_native_must_validate_invalid(self):
        field = self.Field()
        field.field_name = 'field'
        field.parent = mock.MagicMock(must_validate_fields=['field'])

        with self.assertRaises(serializers.ValidationError):
            field.to_internal_value('hello')

    def test_field_from_native_must_validate_ignore(self):
        field = self.Field()
        field.field_name = 'field'
        field.parent = mock.MagicMock(must_validate_fields=[])

        with self.assertRaises(serializers.SkipField):
            field.to_internal_value('hello')


class TestUtils(unittest.TestCase):
    def test_add_base_class_to_instance(self):
        obj = fields.IntegerField(max_value=100)

        self.assertNotIsInstance(obj, EnforceValidationFieldMixin)

        new_obj = add_base_class_to_instance(obj, EnforceValidationFieldMixin)

        self.assertIsInstance(new_obj, EnforceValidationFieldMixin)
        self.assertEqual(vars(obj), vars(new_obj))

    def test__create_enforce_validation_serializer_instance(self):
        new_serializer_class = _create_enforce_validation_serializer(
            TestManySerializer, strict_mode_by_default=False
        )
        new_serializer = new_serializer_class(data={
            'many': [
                {
                    'inner': {
                        'field': '5',
                        'field2': 'hello',
                    },
                    'field': 'hello',
                    'field2': 'world',
                },
            ]
        })

        self.assertIsInstance(
            new_serializer.fields['many'].child.fields['field'],
            EnforceValidationFieldMixin
        )
        self.assertIsInstance(
            new_serializer.fields['many'].child.fields['inner'].fields['field'],
            EnforceValidationFieldMixin
        )
        self.assertListEqual(
            new_serializer.fields['many'].child.must_validate_fields,
            []
        )
        self.assertEqual(
            new_serializer.fields['many'].child.fields['inner'].must_validate_fields,
            ['field']
        )

        self.assertTrue(new_serializer.is_valid(), new_serializer.errors)
        self.assertDictEqual(new_serializer.validated_data, {
            'many': [
                {
                    'inner': {
                        'field': 5,
                    }
                }
            ]
        })

    @mock.patch(TESTING_MODULE + '._create_enforce_validation_serializer')
    def test_create_enforce_validation_serializer_direct_decorator(
            self, mock_create_enforce_validation_serializer):
        @create_enforce_validation_serializer
        class OtherSerializer(TestSerializer):
            pass

        self.assertEqual(OtherSerializer, mock_create_enforce_validation_serializer.return_value)
        mock_create_enforce_validation_serializer.assert_called_once_with(mock.ANY)

    @mock.patch(TESTING_MODULE + '._create_enforce_validation_serializer')
    def test_create_enforce_validation_serializer_decorator_params(
            self, mock_create_enforce_validation_serializer):
        @create_enforce_validation_serializer(foo='bar')
        class OtherSerializer(TestSerializer):
            pass

        self.assertEqual(OtherSerializer, mock_create_enforce_validation_serializer.return_value)
        mock_create_enforce_validation_serializer.assert_called_once_with(mock.ANY, foo='bar')

    def test_create_enforce_validation_serializer_invalid(self):
        with self.assertRaises(TypeError):
            create_enforce_validation_serializer(5)



from __future__ import print_function, unicode_literals
import json
from collections import OrderedDict

import six
from django.conf import settings
from rest_framework import parsers


class SortedJSONParser(parsers.JSONParser):
    """
    Parses JSON-serialized data into OrderedDict.
    """

    def parse(self, stream, media_type=None, parser_context=None):
        """
        Parses the incoming bytestream as JSON and returns the resulting data.
        """
        parser_context = parser_context or {}
        encoding = parser_context.get('encoding', settings.DEFAULT_CHARSET)

        try:
            data = stream.read().decode(encoding)
            return json.loads(data, object_pairs_hook=OrderedDict)
        except ValueError as exc:
            raise parsers.ParseError('JSON parse error - %s' % six.text_type(exc))


# TODO Create a Renderer that does the opposite of this parser.
class StrippingJSONParser(parsers.JSONParser):
    """
    Strip the outer layer of JSON, returning only inner layer.

    This is a convenience class, so that API creators do not need
    to wrap their serializers in a "parent serializer" for the sole
    purpose of stripping out the top-level node.

    Place desired root into parser-context as 'parse-root'.
    Only supports descending one level of nesting.

    Caller is expected to add 'parse_root' to the parser's
    context; a convenient place to do this is in a GenericApiView
    subclass's `get_parser_context()` method.

    Example, for parse_root of "dt_application"::

        input json:
            {
                "dt_application": {
                    "node1": 1234
                }
            }
        output dictionary:
            {
                "node1": 1234
            }
    """

    def parse(self, stream, media_type=None, parser_context=None):
        data = super(StrippingJSONParser, self).parse(
            stream, media_type=media_type, parser_context=parser_context
        )

        try:
            root = parser_context.pop('parse_root')
        except KeyError:
            pass
        else:
            if root in data:
                return data[root]

        return data



from __future__ import print_function, unicode_literals
import unittest
from collections import OrderedDict

import mock
import pytz

from drf_braces.fields.custom import (
    NonValidatingChoiceField,
    PositiveIntegerField,
    UTCDateTimeField,
    UnvalidatedField,
)


class TestUnvalidatedField(unittest.TestCase):
    def test_run_validators(self):
        validator = mock.MagicMock()
        field = UnvalidatedField(validators=[validator])

        actual = field.run_validators(mock.sentinel.value)

        self.assertIsNone(actual)
        self.assertFalse(validator.called)


class TestPositiveIntegerField(unittest.TestCase):
    def test_init_default(self):
        field = PositiveIntegerField()
        self.assertEqual(field.min_value, 0)

    def test_init_passed(self):
        field = PositiveIntegerField(min_value=mock.sentinel.min_value)
        self.assertEqual(field.min_value, mock.sentinel.min_value)


class TestUTCDateTimeField(unittest.TestCase):
    def test_init(self):
        field = UTCDateTimeField()

        self.assertEqual(field.default_timezone, pytz.UTC)


class TestNonValidatingChoiceField(unittest.TestCase):
    def test_init(self):
        field = NonValidatingChoiceField()

        self.assertEqual(field.choices, OrderedDict())

    def test_to_internal_value(self):
        field = NonValidatingChoiceField(choices=['bar'])

        self.assertEqual(field.to_internal_value('bar'), 'bar')
        self.assertEqual(field.to_internal_value('haha'), 'haha')



from __future__ import print_function, unicode_literals

import six
from rest_framework.fields import empty


class EmptyStringFieldMixin(object):
    def validate_empty_values(self, data):
        is_empty, data = super(EmptyStringFieldMixin, self).validate_empty_values(data)
        if not is_empty and data == '':
            if self.required:
                self.fail('required')
            else:
                return True, data
        return is_empty, data

    def to_representation(self, value):
        if value in ('', None) and not self.required:
            return value
        return super(EmptyStringFieldMixin, self).to_representation(value)


class AllowBlankFieldMixin(object):
    def __init__(self, *args, **kwargs):
        if not kwargs.get('required', True):
            kwargs.setdefault('allow_blank', True)
            kwargs.setdefault('allow_null', True)
        super(AllowBlankFieldMixin, self).__init__(*args, **kwargs)


class ValueAsTextFieldMixin(object):
    def to_string_value(self, data):
        if data:
            return six.text_type(data)
        return data

    def prepare_value_for_validation(self, data):
        return data

    def run_validation(self, value=empty):
        (is_empty_value, value) = self.validate_empty_values(value)
        if is_empty_value:
            return value

        value = self.prepare_value_for_validation(value)
        value = self.to_internal_value(value)
        self.run_validators(value)
        value = self.to_string_value(value)

        return value



from __future__ import print_function, unicode_literals
import unittest

import mock
from rest_framework import fields

from drf_braces.fields.mixins import (
    AllowBlankFieldMixin,
    EmptyStringFieldMixin,
    ValueAsTextFieldMixin,
)


class TestEmptyStringFieldMixin(unittest.TestCase):
    def setUp(self):
        super(TestEmptyStringFieldMixin, self).setUp()

        class Field(EmptyStringFieldMixin, fields.IntegerField):
            pass

        self.field = Field()

    def test_validate_empty_values_empty_string_required(self):
        with self.assertRaises(fields.ValidationError):
            self.field.validate_empty_values('')

    def test_validate_empty_values_empty_string(self):
        self.field.required = False

        actual = self.field.validate_empty_values('')

        self.assertTupleEqual(actual, (True, ''))

    def test_validate_empty_values(self):
        self.field.required = False
        self.field.allow_null = True

        actual = self.field.validate_empty_values(None)

        self.assertTupleEqual(actual, (True, None))

    def test_representation(self):
        self.field.required = False

        self.assertEqual(self.field.to_representation('50'), 50)
        self.assertEqual(self.field.to_representation(''), '')

    def test_representation_required(self):
        self.field.required = True

        self.assertEqual(self.field.to_representation('50'), 50)
        with self.assertRaises(ValueError):
            self.field.to_representation('')


class TestAllowBlankFieldMixin(unittest.TestCase):
    def setUp(self):
        super(TestAllowBlankFieldMixin, self).setUp()

        class Field(AllowBlankFieldMixin, fields.CharField):
            pass

        self.field_class = Field

    def test_init(self):
        field = self.field_class(required=False)

        self.assertTrue(field.allow_blank)
        self.assertTrue(field.allow_null)


class TestValueAsTextFieldMixin(unittest.TestCase):
    def setUp(self):
        super(TestValueAsTextFieldMixin, self).setUp()

        class Field(ValueAsTextFieldMixin, fields.IntegerField):
            pass

        self.field = Field(required=False, allow_null=True, max_value=100)

    def test_to_string_value(self):
        self.assertIsNone(self.field.to_string_value(None))
        self.assertEqual(self.field.to_string_value(5), '5')

    def test_prepare_value_for_validation(self):
        self.assertEqual(
            self.field.prepare_value_for_validation(mock.sentinel.value),
            mock.sentinel.value
        )

    def test_run_validation(self):
        self.assertIsNone(self.field.run_validation(None))

        self.assertEqual(self.field.run_validation(50), '50')
        with self.assertRaises(fields.ValidationError):
            self.field.run_validation(500)



from __future__ import print_function, unicode_literals
import unittest

from drf_braces.fields._fields import (
    CharField,
    DateField,
    DateTimeField,
    DecimalField,
    IntegerField,
    RegexField,
    TimeField,
)
from drf_braces.fields.mixins import AllowBlankFieldMixin, EmptyStringFieldMixin


class TestFields(unittest.TestCase):
    def test_subclasses(self):
        fields = [
            DecimalField,
            IntegerField,
            DateTimeField,
            DateField,
            TimeField,
        ]
        for f in fields:
            self.assertTrue(issubclass(f, EmptyStringFieldMixin))

        fields = [
            CharField,
            RegexField,
        ]
        for f in fields:
            self.assertTrue(issubclass(f, AllowBlankFieldMixin))



from __future__ import print_function, unicode_literals
import unittest
from decimal import Decimal

import pytz

from drf_braces.fields.modified import BooleanField, DateTimeField, DecimalField


class TestBooleanField(unittest.TestCase):
    def test_init(self):
        field = BooleanField(true_values=['Y', 'Yes'], false_values=['N', 'No'])
        self.assertIn('Y', field.TRUE_VALUES)
        self.assertIn('Yes', field.TRUE_VALUES)
        self.assertIn('N', field.FALSE_VALUES)
        self.assertIn('No', field.FALSE_VALUES)


class TestDecimalField(unittest.TestCase):
    def test_init(self):
        field = DecimalField()
        self.assertIsNone(field.max_digits)
        self.assertIsNone(field.decimal_places)

    def test_quantize(self):
        field = DecimalField()
        self.assertIsNone(field.quantize(None))

        field = DecimalField(max_digits=4, decimal_places=3)
        self.assertEqual(field.quantize(Decimal('5.1234567')), Decimal('5.123'))


class TestDateTimeField(unittest.TestCase):
    def test_init(self):
        field = DateTimeField()
        self.assertIsNone(field.default_timezone)

        field = DateTimeField(default_timezone=pytz.utc)
        self.assertEqual(field.default_timezone, pytz.utc)



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
import os

import django


DEBUG = True

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'drf-braces.sqlite',
    }
}

INSTALLED_APPS = [
    'drf_braces',
    'tests',

    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django_extensions',
]

MIDDLEWARE_CLASSES = []

STATIC_URL = '/static/'
SECRET_KEY = 'foo'

ROOT_URLCONF = 'tests.urls'



from __future__ import print_function, unicode_literals

import six
from dateutil.parser import parse
from django import forms
from rest_framework import ISO_8601


class ISO8601DateTimeField(forms.DateTimeField):
    def to_python(self, value):
        if value in self.empty_values:
            return

        if isinstance(value, six.string_types) and ISO_8601 in self.input_formats:
            try:
                return parse(value)
            except ValueError:
                raise forms.ValidationError(self.error_messages['invalid'], code='invalid')

        return super(ISO8601DateTimeField, self).to_python(value)



from __future__ import print_function, unicode_literals
import unittest
from collections import OrderedDict
from datetime import datetime

import mock
import six
from django import forms
from rest_framework import fields, serializers

from drf_braces.serializers.form_serializer import (
    FormSerializer,
    FormSerializerBase,
    FormSerializerFieldMixin,
    FormSerializerMeta,
    FormSerializerOptions,
    LazyLoadingValidationsMixin,
    make_form_serializer_field,
)


TESTING_MODULE = 'drf_braces.serializers.form_serializer'


class TestForm(forms.Form):
    foo = forms.CharField(max_length=12)
    bar = forms.IntegerField(max_value=500)
    happy = forms.ChoiceField(required=False)
    other = forms.DateTimeField()

    def __init__(self, *args, **kwargs):
        super(TestForm, self).__init__(*args, **kwargs)
        self.fields['happy'].choices = [
            ('happy', 'choices'),
        ]

    def clean(self):
        data = super(TestForm, self).clean()
        data.update({
            'bar': 257,
        })
        return data


class TestFormSerializerFieldMixin(unittest.TestCase):
    def setUp(self):
        super(TestFormSerializerFieldMixin, self).setUp()

        class Field(FormSerializerFieldMixin, fields.IntegerField):
            pass

        class Serializer(serializers.Serializer):
            field = Field()

            class Meta(object):
                minimum_required = []
                failure_mode = 'fail'

        self.serializer = Serializer()
        self.field = self.serializer.fields['field']

    def test_run_validation_invalid(self):
        with self.assertRaises(fields.ValidationError):
            self.field.run_validation('a')

    def test_run_validation_invalid_failure_mode(self):
        self.serializer.partial = True

        with self.assertRaises(fields.ValidationError):
            self.field.run_validation('a')

    def test_run_validation_invalid_required_field(self):
        self.serializer.partial = True
        self.serializer.Meta.failure_mode = 'drop'
        self.serializer.Meta.minimum_required = ['field']

        with self.assertRaises(fields.ValidationError):
            self.field.run_validation('a')

    def test_run_validation_skip(self):
        self.serializer.partial = True
        self.serializer.Meta.failure_mode = 'drop'
        self.serializer.Meta.minimum_required = []

        with self.assertRaises(fields.SkipField):
            self.field.run_validation('a')


class TestUtils(unittest.TestCase):
    def test_make_form_serializer_field(self):
        field_class = make_form_serializer_field(fields.IntegerField)

        self.assertEqual(field_class.__name__, 'IntegerFormSerializerField')
        self.assertTrue(issubclass(field_class, FormSerializerFieldMixin))


class TestFormSerializerOptions(unittest.TestCase):
    def test_init(self):
        meta = mock.Mock(failure_mode='fail')

        options = FormSerializerOptions(meta, 'foo')

        self.assertEqual(options.form, meta.form)
        self.assertEqual(options.failure_mode, 'fail')
        self.assertEqual(options.minimum_required, meta.minimum_required)
        self.assertEqual(options.field_mapping, options.field_mapping)

    def test_init_invalid(self):
        with self.assertRaises(AssertionError):
            FormSerializerOptions(mock.Mock(failure_mode='foo'), 'foo')
        with self.assertRaises(AssertionError):
            FormSerializerOptions(mock.Mock(form=None), 'foo')
        with self.assertRaises(NotImplementedError):
            FormSerializerOptions(mock.Mock(failure_mode='ignore'), 'foo')


class TestFormSerializerMeta(unittest.TestCase):
    @mock.patch(TESTING_MODULE + '.FormSerializerOptions')
    def test_no_parents(self, mock_form_serializer_options):
        class TestSerializer(six.with_metaclass(FormSerializerMeta, serializers.Serializer)):
            pass

        self.assertFalse(mock_form_serializer_options.called)

    def test_no_meta(self):
        with self.assertRaises(AssertionError):
            class TestSerializer(six.with_metaclass(FormSerializerMeta, FormSerializer)):
                pass

    def test_new(self):
        class TestSerializer(six.with_metaclass(FormSerializerMeta, FormSerializer)):
            class Meta(object):
                form = TestForm

        self.assertIsInstance(TestSerializer.Meta, FormSerializerOptions)
        self.assertIs(TestSerializer.Meta.form, TestForm)


class TestFormSerializerBase(unittest.TestCase):
    def setUp(self):
        super(TestFormSerializerBase, self).setUp()

        class Serializer(FormSerializerBase):
            other = fields.CharField()

            class Meta(object):
                failure_mode = 'drop'
                form = TestForm
                minimum_required = ['foo']
                field_mapping = {}

        self.serializer_class = Serializer

    def test_init(self):
        serializer = self.serializer_class()

        self.assertTrue(serializer.partial)

    def test_get_form(self):
        serializer = self.serializer_class()

        form = serializer.get_form()

        self.assertIsInstance(form, TestForm)
        self.assertIn('foo', form.fields)
        self.assertIn('bar', form.fields)
        self.assertIn('other', form.fields)
        self.assertTrue(form.fields['foo'].required)
        self.assertFalse(form.fields['bar'].required)
        self.assertFalse(form.fields['other'].required)

    def test_get_fields(self):
        serializer = self.serializer_class()
        serializer.Meta.field_mapping.update({
            forms.CharField: fields.BooleanField,
        })

        serializer_fields = serializer.get_fields()

        self.assertIsInstance(serializer_fields, OrderedDict)
        self.assertIn('foo', serializer_fields)
        self.assertIn('bar', serializer_fields)
        self.assertIn('other', serializer_fields)
        self.assertIsInstance(serializer_fields['foo'], fields.BooleanField)
        self.assertIsInstance(serializer_fields['bar'], fields.IntegerField)
        self.assertIsInstance(serializer_fields['other'], fields.CharField)

    def test_get_fields_not_mapped(self):
        serializer = self.serializer_class()

        class FooField(forms.Field):
            pass

        class FooForm(TestForm):
            stuff = FooField()

        serializer.Meta.form = FooForm

        with self.assertRaises(TypeError):
            serializer.get_fields()

    def test_get_field(self):
        serializer = self.serializer_class()
        form_field = forms.ChoiceField(
            choices=[('foo', 'bar')],
            required=False,
            validators=[mock.sentinel.validator],
        )

        field = serializer._get_field(form_field, fields.ChoiceField)

        self.assertIsInstance(field, fields.ChoiceField)
        self.assertTrue(field.allow_blank)
        self.assertTrue(field.allow_null)
        self.assertListEqual(field.validators, [mock.sentinel.validator])
        self.assertDictEqual(field.choice_strings_to_values, {
            'foo': 'foo',
        })

    def test_get_field_kwargs(self):
        serializer = self.serializer_class()
        form_field = forms.IntegerField(
            max_value=500,
            initial=100,
            required=True,
            validators=[mock.sentinel.validator],
        )

        kwargs = serializer._get_field_kwargs(form_field, fields.IntegerField)

        self.assertDictContainsSubset({
            'default': 100,
            'validators': [mock.sentinel.validator, mock.ANY],
        }, kwargs)
        self.assertNotIn('required', kwargs)

    def test_validate(self):
        serializer = self.serializer_class(data={
            'foo': 'hello',
            'bar': '100',
            'other': 'stuff',
        })

        self.assertTrue(serializer.is_valid())
        self.assertDictEqual(serializer.validated_data, {
            'other': 'stuff',
            'foo': 'hello',
            'bar': 257,
            'happy': '',
        })

    def test_validate_valid(self):
        serializer = self.serializer_class(data={
            'foo': 'hello',
            'bar': '100',
            'other': '2015-01-01 12:30',
        })

        self.assertTrue(serializer.is_valid())
        self.assertDictEqual(serializer.validated_data, {
            'other': datetime(2015, 1, 1, 12, 30),
            'foo': 'hello',
            'bar': 257,
            'happy': '',
        })

    def test_validate_fail(self):
        self.serializer_class.Meta.failure_mode = 'fail'
        serializer = self.serializer_class(data={
            'foo': 'hello',
            'bar': '100',
            'other': 'stuff',
        })

        self.assertFalse(serializer.is_valid())
        self.assertDictEqual(serializer.errors, {
            'other': ['Enter a valid date/time.'],
        })

    def test_to_representation(self):
        with self.assertRaises(NotImplementedError):
            self.serializer_class().to_representation({})


class TestFormSerializer(unittest.TestCase):
    def test_bases(self):
        self.assertTrue(issubclass(FormSerializer, FormSerializerBase))
        self.assertIsInstance(FormSerializer, FormSerializerMeta)


class TestLazyLoadingValidationsMixin(unittest.TestCase):
    def setUp(self):
        super(TestLazyLoadingValidationsMixin, self).setUp()

        class Serializer(LazyLoadingValidationsMixin, FormSerializer):
            class Meta(object):
                form = TestForm

        self.serializer_class = Serializer

    def test_repopulate_form_fields(self):
        serializer = self.serializer_class()

        # sanity check
        self.assertDictEqual(serializer.fields['happy'].choices, {})

        serializer.repopulate_form_fields()

        self.assertDictEqual(serializer.fields['happy'].choices, {'happy': 'choices'})
        self.assertDictEqual(serializer.fields['happy'].choice_strings_to_values,
                             {'happy': 'happy'})

    @mock.patch.object(serializers.Serializer, 'to_internal_value')
    @mock.patch.object(LazyLoadingValidationsMixin, 'repopulate_form_fields')
    def test_to_internal_value(self, mock_repopulate_form_fields, mock_super_to_internal_value):
        serializer = self.serializer_class()

        serializer.to_internal_value({})

        mock_repopulate_form_fields.assert_called_once_with()



from __future__ import print_function, unicode_literals
import inspect
import itertools


IGNORE_ARGS = ['self', 'cls']


def find_function_args(func):
    """
    Get the list of parameter names which function accepts.
    """
    try:
        spec = inspect.getargspec(func)
        return [i for i in spec[0] if i not in IGNORE_ARGS]
    except TypeError:
        return []


def find_class_args(klass):
    """
    Find all class arguments (parameters) which can be passed in ``__init__``.
    """
    args = set()

    for i in klass.mro():
        if i is object or not hasattr(i, '__init__'):
            continue
        args |= set(find_function_args(i.__init__))

    return list(args)


def find_matching_class_kwargs(reference_object, klass):
    return {
        i: getattr(reference_object, i) for i in find_class_args(klass)
        if hasattr(reference_object, i)
    }


def add_base_class_to_instance(instance, base_class=None, new_name=None):
    """
    Generic utility for adding a base class to an instance.

    This function returns a copy of the given instance which
    will then include the new base_class in its ``__mro__``.

    The way that is done internally is it creates a brand new
    class with correct bases. Then the newly created class is
    instantiated. Since ``__init__`` could be expensive operation
    in any of the base classes of the original instance mro,
    nto make it cheap, we temporarily switch __init__ with
    super simple implementation which does nothing but only
    instantiates class. Once instantiated, then we copy all of the
    instance attributes to the newly created instance.
    Finally, then we pop our mock ``__init__`` implementation.

    Args:
        instance (object): Instance of any object
        base_class (type): Any class which will be added as first class
            in the newly copied instance mro.

    Returns:
        Shallow copy of ``instance`` which will also inherit ``base_class``.
    """
    # overwrite __init__ since that is mainly responsible for setting
    # instance state but since we explicitly copy it, we can
    # make __init__ a noop method
    def __init__(self, *args, **kwargs):
        pass

    if base_class is not None and base_class not in instance.__class__.mro():
        base_classes = (base_class, instance.__class__)
    else:
        base_classes = (instance.__class__,)

    new_field_class = type(
        str(new_name or instance.__class__.__name__),
        base_classes,
        {'__init__': __init__}
    )

    new_instance = new_field_class()
    new_instance.__dict__.update(instance.__dict__)

    # we added __init__ just for faster instantiation
    # since then we dont have to copy all the parameters
    # when creating new instance and then update its state
    # however after we instantiated the class, we want to
    # pop our silly __init__ implementation so that if somebody
    # wants to instantiate instance.__class__(), it will
    # use the original __init__ method
    del new_field_class.__init__

    return new_instance


def initialize_class_using_reference_object(reference_object, klass, **kwargs):
    """
    Utility function which instantiates ``klass`` by extracting ``__init__``
    kwargs from ``reference_object`` attributes.

    Args:
        reference_object (object): Any object instance from which matching
            attributes will be used as ``klass``'s ``__init__`` kwargs.
        klass (type): Class which will be instantiated by using
            ``reference_object`` attributes.
        **kwargs: Any additional kwargs which will be passed during instantiation.

    Returns:
        Instantiated ``klass`` object.
    """
    _kwargs = find_matching_class_kwargs(reference_object, klass)
    _kwargs.update(kwargs)

    return klass(**_kwargs)


def get_class_name_with_new_suffix(klass, existing_suffix, new_suffix):
    class_name = klass.__name__

    if existing_suffix in class_name:
        prefix, suffix = class_name.rsplit(existing_suffix, 1)
    else:
        prefix, suffix = class_name, ''

    new_name = str('{}{}{}'.format(prefix, new_suffix, suffix))

    return new_name


def get_attr_from_base_classes(bases, attrs, attr, **kwargs):
    if attr in attrs:
        return attrs[attr]

    for base in bases:
        try:
            return getattr(base, attr)
        except AttributeError:
            continue

    if 'default' in kwargs:
        return kwargs['default']

    raise AttributeError(
        'None of the bases have {} attribute'
        ''.format(attr)
    )


def reduce_attr_dict_from_base_classes(bases, getter, default=None):
    data = (default or {}).copy()

    # get all field mappings from super methods
    for base in itertools.chain(*[reversed(i.mro()) for i in reversed(bases)]):
        data.update(getter(base) or {})

    return data


def reduce_attr_dict_from_instance(self, getter, default=None):
    data = reduce_attr_dict_from_base_classes(type(self).mro(), getter, default)

    # this should of been picked by reduce_attr_dict_from_base_classes
    # however that only accounts base classes and not instance
    # attribute which can be modified in __init__, etc so we
    # explicitly account for it
    data.update(getter(self) or {})

    return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
import os
import sys


sys.path.insert(0, os.path.abspath(os.path.join(__file__, '..', '..')))
sys.path.insert(0, os.getcwd())

if __name__ == "__main__":
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "tests.settings")

    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



from __future__ import print_function, unicode_literals
import inspect

from rest_framework import fields
from rest_framework.fields import *  # noqa
from rest_framework.fields import _UnvalidatedField  # noqa

from .mixins import AllowBlankFieldMixin, EmptyStringFieldMixin


EMPTY_STRING_FIELDS = [
    fields.DecimalField,
    fields.IntegerField,
    fields.DateTimeField,
    fields.DateField,
    fields.TimeField,
]

ALLOW_BLANK_FIELDS = [
    fields.CharField,
    fields.RegexField,
]


def get_updated_fields(fields, base_class):
    return {
        field.__name__: type(field.__name__, (base_class, field), {})
        for field in fields
    }


locals().update(get_updated_fields(EMPTY_STRING_FIELDS, EmptyStringFieldMixin))
locals().update(get_updated_fields(ALLOW_BLANK_FIELDS, AllowBlankFieldMixin))

__all__ = [name for name, value in locals().items()
           if inspect.isclass(value) and issubclass(value, fields.Field)]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
import os

from setuptools import find_packages, setup

from drf_braces import __author__, __version__


def read(fname):
    return (open(os.path.join(os.path.dirname(__file__), fname), 'rb')
            .read().decode('utf-8'))


authors = read('AUTHORS.rst')
history = read('HISTORY.rst').replace('.. :changelog:', '')
licence = read('LICENSE.rst')
readme = read('README.rst')

requirements = read('requirements.txt').splitlines() + [
    'setuptools',
]

test_requirements = (
    read('requirements.txt').splitlines()
    + read('requirements-dev.txt').splitlines()[1:]
)

setup(
    name='django-rest-framework-braces',
    version=__version__,
    author=__author__,
    description='Collection of utilities for working with DRF',
    long_description='\n\n'.join([readme, history, authors, licence]),
    url='https://github.com/dealertrack/django-rest-framework-braces',
    license='MIT',
    packages=find_packages(exclude=['tests', 'tests.*']),
    install_requires=requirements,
    test_suite='tests',
    tests_require=test_requirements,
    keywords=' '.join([
        'django-rest-framework',
    ]),
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Development Status :: 2 - Pre-Alpha',
    ],
)
